// COPYRIGHT (C) 1990-1998.  All Rights Reserved. - BITMAP5.CPP
// Zinc Software Incorporated.  Pleasant Grove, Utah  USA
// May be freely copied, used and distributed.

#include <zaf.hpp>
#include <z_utils.hpp>
#include <z_prst.hpp>
#include "bitmap2.hpp"
#include "bitmap4.hpp"
#include "bitmap5.hpp"

// Callback for the slider.
ZafEventType ChangeInterval(ZafWindowObject *, ZafEventStruct &, ZafEventType ccode)
{
	if (ccode == L_SELECT)
		zafEventManager->Put(U_ADJUST_TIMER);

	return (0);
}

// Constructor loads the window from the persistence file.
ControlWindow::ControlWindow(ZafIChar *name) : ZafWindow(name, *zafObjectPersistence),
bitmapCounter(0)
{
	// Initialize my protected members.
	slider = DynamicPtrCast(GetObject(ZAF_ITEXT("SLIDER")), ZafScrollBar);
	slider->SetUserFunction(ChangeInterval);
	bitmapCombo = DynamicPtrCast(GetObject(ZAF_ITEXT("ACTIVE_BITMAP")), ZafComboBox);

	currentInterval = DynamicPtrCast(GetObject(ZAF_ITEXT("CURRENT_INTERVAL")), ZafInteger);
	zafWindowManager->Center(this);
}

// Destructor cleans up any of its own data.
ControlWindow::~ControlWindow()
{
}

// All events are handled in the Event function.
ZafEventType ControlWindow::Event(const ZafEventStruct &event)
{
	ZafEventType ccode = event.type;
	switch (ccode)
	{
	case U_ADJUST_TIMER:
		ResetTimerInterval();
		break;

	case U_OPEN_BITMAP:
		OpenBitmapWindow();
		break;

	case U_CLOSE_BITMAP:
		CloseBitmapWindow();
		break;

	case U_UPDATE_SCROLL:
		ResetScrollBar();
		break;

	default:
		ccode = ZafWindow::Event(event);
	}
	return (ccode);
}

// Open a new ZafAnimatedBitmap window.
void ControlWindow::OpenBitmapWindow(void)
{
	bitmapCounter++;
	ZafIChar buff[ZAF_MAXNAMELEN];
	sprintf(buff, ZAF_ITEXT("Window %i"), bitmapCounter);

	ZafButton *button = new ZafButton(0, 0, 0, 0, buff);
	button->SetSendMessageWhenSelected(true);
	button->SetValue(U_UPDATE_SCROLL);
	bitmapCombo->Add(button);
	button->SetSelected(true);
	button->SetButtonType(ZAF_FLAT_BUTTON);

	// Create the window and add the support objects.
	ZafWindow *window = new ZafWindow(0, 0, 25, 4);
	window->AddGenericObjects(new ZafStringData(buff));
	window->SetStringID(buff);
	window->SetLocked(true);

	// Add the animated bitmap object to the window.
	ZafAnimatedBitmap *bitmap = new ZafAnimatedBitmap(4, 1, 0, 0,
		ZAF_ITEXT("WORLD_"), 15, MIN_INTERVAL);
	bitmap->SetBordered(false);
	bitmap->SetStringID(ZAF_ITEXT("Bitmap"));
	window->Add(bitmap);

	// Set the integer and scrollbar data.
	ZafIntegerData *intervalData = currentInterval->IntegerData();
	intervalData->SetInteger(MIN_INTERVAL);
	slider->ScrollData()->SetCurrent(MAX_INTERVAL);

	zafWindowManager->Add(window);
}

// Close the window shown in the combo.
void ControlWindow::CloseBitmapWindow(void)
{
	ZafIChar *currentBitmap = ZAF_NULLP(ZafIChar);
	ZafWindowObject *window = ZAF_NULLP(ZafWindow);

	ZafWindowObject *object = bitmapCombo->Current();
	if (object)
	{
		currentBitmap = strdup(object->Text());
		// Make the next or previous object current in the combo.
		ZafWindowObject *prevObject = object->Next() ?
			object->Next() : object->Previous();

		bitmapCombo->Subtract(object);
		delete object;

		if (prevObject)
			prevObject->SetSelected(true);
	}
	if (currentBitmap)
	{
		window = zafWindowManager->GetObject(currentBitmap);
		delete currentBitmap;
	}
	if (window)
	{
		zafWindowManager->Subtract(window);
		delete window;
	}
	return;
}

// Reset the timer for the window shown in the combo.
void ControlWindow::ResetTimerInterval(void)
{
	// Set up the stringID to get the bitmap object.
	ZafIChar buff[ZAF_MAXNAMELEN];
	sprintf(buff, ZAF_ITEXT("%s%s"), bitmapCombo->Text(), ZAF_ITEXT("~Bitmap"));

	// Get the current slider information.
	int interval = (int)slider->ScrollData()->Current();

	// Set the integer information.
	ZafIntegerData *intervalData = currentInterval->IntegerData();
	intervalData->SetInteger(MAX_INTERVAL - interval + MIN_INTERVAL);

	// Set the timer interval on the animated bitmap.
	ZafAnimatedBitmap *bitmap = 
		DynamicPtrCast(zafWindowManager->GetObject(buff), ZafAnimatedBitmap);
	if (bitmap)
		bitmap->SetTimerInterval(MAX_INTERVAL - interval + MIN_INTERVAL);

	return;
}

// Reset the timer for the window shown in the combo.
void ControlWindow::ResetScrollBar()
{
	// Set up the stringID to get the bitmap object.
	ZafIChar buff[ZAF_MAXNAMELEN];
	sprintf(buff, ZAF_ITEXT("%s%s"), bitmapCombo->Text(), ZAF_ITEXT("~Bitmap"));

	// Get a pointer to the bitmap from the selected window.
	ZafAnimatedBitmap *bitmap =
		DynamicPtrCast(zafWindowManager->GetObject(buff), ZafAnimatedBitmap);

	// Make sure the object is valid.
	if (!bitmap)
		return;

	// Get the timer interval for the selected bitmap window.
	long interval = bitmap->TimerInterval();

	// Reset the scroll information.
	ZafScrollData *scrollData = slider->ScrollData();
	scrollData->SetCurrent(MAX_INTERVAL - interval + MIN_INTERVAL);

	// Reset the integer information.
	ZafIntegerData *intervalData = currentInterval->IntegerData();
	intervalData->SetInteger(interval);

	return;
}
