// COPYRIGHT (C) 1998.  All Rights Reserved. - REAL.CPP
// Zinc Software Incorporated.  Pleasant Grove, Utah  USA
// May be freely copied, used and distributed.

#include <zaf.hpp>
#include <z_prst.hpp>
#include "real.hpp"

// ----- FormatWindow -------------------------------------------------------

FormatWindow::FormatWindow(ZafIChar *name) :
	ZafWindow(name, *zafObjectPersistence)
{
	// Check for read errors.
	if (Error())
		return;

	// Create a ZafRealData to be shared by all ZafReals.
	ZafRealData *realData = new ZafRealData();

	// Get a pointer to each ZafReal and set its ZafRealData to be 
	// the one just created.
	editReal = DynamicPtrCast(GetObject(ZAF_ITEXT("EnterRealData")), ZafReal);
	editReal->SetRealData(realData);
	
	ZafReal *zReal = DynamicPtrCast(GetObject(ZAF_ITEXT("%f:")), ZafReal);
	zReal->SetRealData(realData);

	zReal = DynamicPtrCast(GetObject(ZAF_ITEXT("%4.3e:")), ZafReal);
	zReal->SetRealData(realData);
	
	zReal = DynamicPtrCast(GetObject(ZAF_ITEXT("%G:")), ZafReal);
	zReal->SetRealData(realData);

	zReal = DynamicPtrCast(GetObject(ZAF_ITEXT("%8.4f:")), ZafReal);
	zReal->SetRealData(realData);

	zReal = DynamicPtrCast(GetObject(ZAF_ITEXT("DefaultFormat:")), ZafReal);
	zReal->SetRealData(realData);

	inputFormat = DynamicPtrCast(GetObject(ZAF_ITEXT("InputFormat")), ZafString);
	inputFormat->SetUserFunction(FormatReal);

	outputFormat = DynamicPtrCast(GetObject(ZAF_ITEXT("OutputFormat")), ZafString);
	outputFormat->SetUserFunction(FormatReal);
}

ZafEventType FormatWindow::Event(const ZafEventStruct &event)
{
	ZafEventType ccode = event.type;
	switch(ccode)
	{
	case CHANGE_INPUT_FORMAT:
	case CHANGE_OUTPUT_FORMAT:
		{
		if (ccode == CHANGE_OUTPUT_FORMAT && strlen(outputFormat->Text()) < 2)
			break;
		// If editReal has focuse set userStatus to 1.
		bool saveFocus = editReal->Focus();
		
		// Get the next object in the list following the editReal so
		// that when it is added back to the window it will remain in the
		// same tab order.
		ZafWindowObject *nextObj = editReal->Next();

		// Subtract the editReal from the window so that its input or 
		// output format can be changed.
		Subtract(editReal);
		if (ccode == CHANGE_OUTPUT_FORMAT)
		{
			// Clear the Changed attribute, get the new format and set it.
			outputFormat->SetChanged(false);
			editReal->SetOutputFormat(outputFormat->Text());
			editReal->SetInputFormat(outputFormat->Text());
		}
		else
		{
			// Clear the Changed attribute, get the new format and set it.
			inputFormat->SetChanged(false);
			editReal->SetInputFormat(outputFormat->Text());
		}

		// Add the editReal back to the window specifying its position.
		Add(editReal, nextObj);

		// If the editReal had focus give it focus and clear userStatus.
		if (saveFocus)
			editReal->SetFocus(true);
		}
		break;

	default:
		ccode = ZafWindow::Event(event);
		break;
	}

	// Return the control code.
	return (ccode);
}

ZafEventType FormatWindow::FormatReal(ZafWindowObject *object,
	ZafEventStruct &, ZafEventType ccode)
{
	// If a format string has been selected or is going non currect and
	// its contents has changed, determine which format string it is and
	// place the correct message on the queue.
	if ((ccode == N_NON_CURRENT || ccode == L_SELECT) && object->Changed())
	{
		if (!strcmp(object->StringID(), ZAF_ITEXT("OutputFormat")))
			object->eventManager->Put(CHANGE_OUTPUT_FORMAT);
		else
			object->eventManager->Put(CHANGE_INPUT_FORMAT);
	}

	// Return the control code.
	return (0);
}

// ----- ZafApplication -----------------------------------------------------

int ZafApplication::Main(void)
{
	LinkMain();

	// Create a storage to be used in creating a storage.
	ZafStorage *storage = new ZafStorage(ZAF_ITEXT("p_real.znc"), ZAF_FILE_READ);

	// Use the zafObjectPersistence global varible and create a 
	// pesistance object.
	zafObjectPersistence = new ZafObjectPersistence(storage,
		zafDefaultDataConstructor, zafDefaultObjectConstructor);

	// Load the main window from the data (.znc) file.
	ZafWindow *window = new FormatWindow(ZAF_ITEXT("MainWindow"));

	// Attach the main window to the window manager, or in other words,
	// make it appear on the screen.
	zafWindowManager->Add(window);

	// Process events.
	// (This functions passes events from the event manager to the
	// window manager until an S_EXIT is received or until there on no
	// more windows attached to the window manager.)
	Control();

	// Clean up the persistence and storage.
	delete zafObjectPersistence;
	zafObjectPersistence = ZAF_NULLP(ZafObjectPersistence);
	delete storage;

	return (0);
}
