// COPYRIGHT (C) 1998.  All Rights Reserved. - INTL3.CPP
// Zinc Software Incorporated.  Pleasant Grove, Utah  USA
// May be freely copied, used and distributed.

#include <zaf.hpp>
#include <z_prst.hpp>

// ----- InternationalWindow ------------------------------------------------

class InternationalWindow : public ZafWindow
{
public:
	InternationalWindow(ZafIChar *name, ZafObjectPersistence &persist);
	ZafEventType Event(const ZafEventStruct &event);

protected:
	ZafWindowObject *i18nSetting;
};

InternationalWindow::InternationalWindow(ZafIChar *name, ZafObjectPersistence &persist) :
	ZafWindow(name, persist)
{
	// Center the window on the display.
	zafWindowManager->Center(this);

	// Determine the current setting.
	i18nSetting = GetObject(ZAF_ITEXT("LanguageString"));
	i18nSetting->SetText(ZafI18nData::I18nName());

	ZafWindowObject *newSetting = GetObject("NewI18nSetting");
	ZafComboBox *comboSetting = newSetting ? DynamicPtrCast(newSetting, ZafComboBox) :
		ZAF_NULLP(ZafComboBox);
	if (comboSetting)
		for (ZafButton *button = DynamicPtrCast(comboSetting->First(), ZafButton);
			button; button = DynamicPtrCast(button->Next(), ZafButton))
			if (!strcmp(button->SendMessageText(), i18nSetting->Text()))
				button->SetSelected(true);
}

ZafEventType InternationalWindow::Event(const ZafEventStruct &event)
{
	// Reset the current setting, based on the i18n information.
	if (event.type == N_RESET_I18N)
		i18nSetting->SetText(ZafI18nData::I18nName());

	// Defer to the base class.
	return (ZafWindow::Event(event));
}

// ----- ZafApplication -----------------------------------------------------

int ZafApplication::Main(void)
{
	LinkMain();

	// Create the window persistence.
	ZafStorage *storage = new ZafStorage(ZAF_ITEXT("intl3.znc"), ZAF_FILE_READ);
	zafObjectPersistence = new ZafObjectPersistence(storage,
		zafDefaultDataConstructor, zafDefaultObjectConstructor);

	// Load the window.
	zafWindowManager->Add(new InternationalWindow(ZAF_ITEXT("International"), *zafObjectPersistence));

	// Get user input.
	Control();
	
	// Destroy the window persistence.
	delete zafObjectPersistence;
	zafObjectPersistence = ZAF_NULLP(ZafObjectPersistence);
	delete storage;

	// Return success.
	return (0);
}

