//	Zinc Application Framework - Z_DSKFIL.CPP
//	COPYRIGHT (C) 1990-1998.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#define ZAF_TEMP_NAME_PREFIX		ZAF_ITEXT("ZTMP")
#define ZAF_TEMP_NAME_EXTENSION		ZAF_ITEXT(".$$$")

#if defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__)
#	include <dir.h>
#	include <direct.h>
#elif defined(__hpux) || defined(__sgi) || defined(sun) || defined(_SINIX) || defined(_IBM_RS6000) || defined(SCO_UNIX) || defined(__QNX__) || defined(__GNUC__) || defined(__DECCXX)
#	include <dirent.h>
#	include <sys/stat.h>
#elif (defined(__SC__) && !defined(macintosh)) || defined(_MSC_VER) || (defined(__WATCOMC__) && !defined(__QNX__)) || defined(__HIGHC__) || defined(__IBMCPP__) || defined(__GLOCKENSPIEL__)
#	include <direct.h>
#endif
#if defined(__SC__) && !defined(macintosh)
#	define _dos_getdrive dos_getdrive
#	define _dos_setdrive dos_setdrive
#endif
#if defined(_MSC_VER) && defined(DOS386)
#	include <pldos32.h>
#endif
#include <z_env.hpp>
#if defined(ZAF_MSDOS) || defined(ZAF_MSWINDOWS) || defined(ZAF_OS2)
#	if !defined(__IBMCPP__) && !defined(__GLOCKENSPIEL__)
#		include <dos.h>
#	endif
#endif

#include <z_stdlib.hpp>
#include <z_ctype.hpp>
#include <z_dskfil.hpp>
#include <z_cset.hpp>
#include <z_string.hpp>
#include <z_unistd.hpp>
#include <z_utils.hpp>

#if defined(ZAF_MACINTOSH)
#	include "a_utils.hpp"

	OSType ZafDiskFile::fileCreator = 'Anon';
	OSType ZafDiskFile::fileType = 'TEXT';
#endif

// ----- ZafDiskFileSystem --------------------------------------------------

void ZafDiskFileSystem::ChangeExtension(ZafIChar *pathName,
	const ZafIChar *newExtension)
{
	int i = strlen(pathName) - 1;
	while (i >= 0 && pathName[i] != '.' && pathName[i] != zafCodeSet->dirSepStr[0])
		--i;

	if (i >= 0 && pathName[i] == '.')
		pathName[i] = '\0';
	if (newExtension)
	{
		if (newExtension[0] != '.' && pathName[strlen(pathName)-1] != '.')
			strcat(pathName, ZAF_ITEXT("."));
		strcat(pathName, newExtension);
	}
}

void ZafDiskFileSystem::MakeFullPath(ZafIChar *fullPath,
	const ZafIChar *pathName, const ZafIChar *fileName,
	const ZafIChar *extension)
{
	// Check for quicker method of passing in the same pointer for fullPath and pathName.
	if (pathName != fullPath)
		strcpy(fullPath, pathName);
	int i = strlen(fullPath);
#if defined(ZAF_MSDOS) || defined(ZAF_MSWINDOWS) || defined(ZAF_OS2)
	if (i && fullPath[i-1] != zafCodeSet->dirSepStr[0] && fullPath[i-1] != ':')
#else
	if (i && fullPath[i-1] != zafCodeSet->dirSepStr[0])
#endif
		strcat(fullPath, zafCodeSet->dirSepStr);
	if (fileName)
		strcat(fullPath, fileName);
	if (extension)
		ChangeExtension(fullPath, extension);
}

void ZafDiskFileSystem::StripFullPath(const ZafIChar *fullPath,
	ZafIChar *pathName, ZafIChar *fileName, ZafIChar *objectName,
	ZafIChar *objectPathName)
{
	// Determine the path/file split area.
	const ZafIChar *endPath = strrchr(fullPath, zafCodeSet->dirSepStr[0]);
#if defined(ZAF_MSDOS) || defined(ZAF_MSWINDOWS) || defined(ZAF_OS2)
	if (!endPath)
		endPath = strrchr(fullPath, ':');
#endif
	if (!endPath)
		endPath = fullPath;
	if (endPath == fullPath && *endPath == zafCodeSet->dirSepStr[0])
		endPath++;
#if defined(ZAF_MSDOS) || defined(ZAF_MSWINDOWS) || defined(ZAF_OS2)
	if (*endPath == ':')
		endPath++;
	if (*endPath == zafCodeSet->dirSepStr[0] && endPath-fullPath == 2 && endPath[-1] == ':')
		endPath++;
#elif defined(ZAF_MOTIF)
	if (endPath == fullPath && *endPath == '~')
		endPath++;
#endif

	// Determine the file/object split area.
	const ZafIChar *endFile = strchr(endPath, ZAF_DIRECTORY_SEPARATOR);
	if (!endFile || endFile == fullPath)
		endFile = &endPath[strlen(endPath)];

	// find the end of the object path name
	const ZafIChar *endOPath = strrchr(endFile, ZAF_DIRECTORY_SEPARATOR);
	if (!endOPath || endOPath == fullPath)
		endOPath = &endFile[strlen(endFile)];

	// Construct the path name.
	if (pathName)
	{
		int i = (int)(endPath - fullPath);
		strncpy(pathName, fullPath, i);
		pathName[i] = '\0';
	}
	if (*endPath == zafCodeSet->dirSepStr[0])
		endPath++;

	// Construct the file name.
	if (fileName)
	{
		int i = (int)(endFile - endPath);
		strncpy(fileName, endPath, i);
		fileName[i] = '\0';
	}

	// Construct the object path name.
	if (objectPathName)
	{
		int i = (int)(endOPath - endFile);
		strncpy(objectPathName, endFile, i);
		objectPathName[i] = '\0';
	}
	if (*endOPath == zafCodeSet->dirSepStr[0])
		endOPath++;

	// Construct the object name.
	if (objectName)
		strcpy(objectName, endOPath);
}

void ZafDiskFileSystem::TempName(ZafIChar *tempName)
{
	ZafIChar pathName[ZAF_MAXPATHLEN];
	strcpy(pathName, tempName);

	int index = 0;
	do
	{
		ZafIChar fileName[16];
		sprintf(fileName, ZAF_ITEXT("%s%d%s"), ZAF_TEMP_NAME_PREFIX, index++,
			ZAF_TEMP_NAME_EXTENSION);
		MakeFullPath(tempName, pathName, fileName);
	} while (ValidName(tempName, false));
}

ZafDiskFileSystem::ZafDiskFileSystem(void) :
	ZafFileSystem()
{
}

ZafDiskFileSystem::~ZafDiskFileSystem(void)
{
}

int ZafDiskFileSystem::ChDir(const ZafIChar *newPath, ZafStringID,
	ZafNumberID)
{
	// Move to the new directory.
#if defined(ZAF_MACINTOSH)
	// Not implemented.
#	pragma unused(newPath)
	return (-1);
#else
	return (chdir(newPath));
#endif
}

int ZafDiskFileSystem::FindFirst(ZafFileInfoStruct &fileInfo,
	const ZafIChar *searchName, ZafStringID searchStringID,
	ZafNumberID searchNumberID)
{
	// Initialize the info information.
	fileInfo.name = ZAF_NULLP(ZafIChar);
	fileInfo.stringID = ZAF_NULLP(ZafIChar);
	fileInfo.numberID = 0;
	fileInfo.length = -1;
	fileInfo.attributes = ZAF_FATTRIB_NONE;
	fileInfo.internalHandle = ZAF_NULLP(void);

	// Find the file.
	return (FindNext(fileInfo, searchName, searchStringID, searchNumberID));
}

int ZafDiskFileSystem::FindNext(ZafFileInfoStruct &fileInfo,
	const ZafIChar *searchName, ZafStringID , ZafNumberID )
{
	// Initialize the search variables.
	int tError = 0;
	char osName[ZAF_MAXPATHLEN];
	zafCodeSet->ConvertToOSString(searchName, osName, false);

	// Prepare for the next lookup.
	if (fileInfo.name)
	{
		delete (ZafIChar *)fileInfo.name;
		fileInfo.name = ZAF_NULLP(ZafIChar);
	}
	fileInfo.length = -1;
	fileInfo.attributes = ZAF_FATTRIB_NONE;

	// Check for find-first implementations.
#if defined(ZAF_WIN32)
	HANDLE *searchHandle = (HANDLE *)fileInfo.internalHandle;
	WIN32_FIND_DATAA findData ;
	if (!searchHandle)
	{
		fileInfo.internalHandle = searchHandle = new HANDLE;
		*searchHandle = FindFirstFileA(osName, &findData);
		if (*searchHandle == INVALID_HANDLE_VALUE)
			tError = -1;
	}
	else if (!FindNextFileA(*searchHandle, &findData))
		tError = -1;

	// Fill the fileInfo structure.
	if (!tError)
	{
		fileInfo.name = zafCodeSet->ConvertToZafString(findData.cFileName);
		fileInfo.length = findData.nFileSizeLow;
		if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			fileInfo.attributes |= ZAF_FATTRIB_DIRECTORY;
		if (findData.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
			fileInfo.attributes |= ZAF_FATTRIB_READONLY;
	}
#elif defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__)
	ffblk *info = (ffblk *)fileInfo.internalHandle;
	if (!info)
	{
		fileInfo.internalHandle = info = new ffblk;
		if (findfirst(osName, info, FA_RDONLY | FA_DIREC) != 0)
			tError = -1;
	}
	else if (findnext(info) != 0)
		tError = -1;

	// Fill the fileInfo structure.
	if (!tError)
	{
		fileInfo.name = zafCodeSet->ConvertToZafString(info->ff_name);
		fileInfo.length = info->ff_fsize;
		if (info->ff_attrib & FA_DIREC)
			fileInfo.attributes |= ZAF_FATTRIB_DIRECTORY;
		if (info->ff_attrib & FA_RDONLY)
			fileInfo.attributes |= ZAF_FATTRIB_READONLY;
	}
#elif defined(_MSC_VER)
	_find_t *info = (_find_t *)fileInfo.internalHandle;
	if (!info)
	{
		fileInfo.internalHandle = info = new _find_t;
		if (_dos_findfirst(osName, _A_RDONLY | _A_SUBDIR, info) != 0)
			tError = -1;
	}
	else if (_dos_findnext(info) != 0)
		tError = -1;

	// Fill the fileInfo structure.
	if (!tError)
	{
		fileInfo.name = zafCodeSet->ConvertToZafString(info->name);
		fileInfo.length = info->size;
		if (info->attrib & _A_SUBDIR)
			fileInfo.attributes |= ZAF_FATTRIB_DIRECTORY;
		if (info->attrib & _A_RDONLY)
			fileInfo.attributes |= ZAF_FATTRIB_READONLY;
	}
#elif defined(__WATCOMC__) && !defined(__QNX__)
	find_t *info = (find_t *)fileInfo.internalHandle;
	if (!info)
	{
		fileInfo.internalHandle = info = new find_t;
		if (_dos_findfirst(osName, _A_SUBDIR, info) != 0)
			tError = -1;
	}
	else if (_dos_findnext(info) != 0)
		tError = -1;

	// Fill the fileInfo structure.
	if (!tError)
	{
		fileInfo.name = zafCodeSet->ConvertToZafString(info->name);
		fileInfo.length = info->size;
		if (info->attrib & _A_SUBDIR)
			fileInfo.attributes |= ZAF_FATTRIB_DIRECTORY;
		if (info->attrib & _A_RDONLY)
			fileInfo.attributes |= ZAF_FATTRIB_READONLY;
	}
#elif defined(ZAF_MOTIF)
	ZafIChar directoryName[ZAF_MAXPATHLEN];
	ZafIChar fileName[ZAF_MAXPATHLEN];
	StripFullPath(searchName, directoryName, fileName);

	DIR *directory = (DIR *)fileInfo.internalHandle;
	if (!directory)
	{
		if (directoryName[0])
		{
			char osDirectoryName[ZAF_MAXPATHLEN];
			zafCodeSet->ConvertToOSString(directoryName, osDirectoryName, false);
			fileInfo.internalHandle = directory = opendir(osDirectoryName);
		}
		else
			fileInfo.internalHandle = directory = opendir(".");
	}

	dirent *info = readdir(directory);
	while (info)
	{
		fileInfo.name = zafCodeSet->ConvertToZafString(info->d_name);		
		struct stat fileStats;
		ZafIChar fullPath[ZAF_MAXPATHLEN];
		MakeFullPath(fullPath, directoryName, fileInfo.name);	
 		zafCodeSet->ConvertToOSString(fullPath, osName, false);
		stat(osName, &fileStats);
		
		if (WildStrcmp(fileInfo.name, fileName) == 0)
		{
			fileInfo.length = fileStats.st_size;
			if (S_ISDIR(fileStats.st_mode))
				fileInfo.attributes |= ZAF_FATTRIB_DIRECTORY;
			if ((fileStats.st_mode & (S_IWUSR | S_IWGRP | S_IWOTH)) == 0)
				fileInfo.attributes |= ZAF_FATTRIB_READONLY;
			break;
		}
		else
		{
			delete (ZafIChar *)fileInfo.name;
			fileInfo.name = ZAF_NULLP(ZafIChar);
		}

		info = readdir(directory);
	}

	if (!info && directory)
	{
		tError = -1;
		closedir(directory);
		fileInfo.internalHandle = ZAF_NULLP(void);
	}
#endif

	// Return the error status.
	return (tError);
}

int ZafDiskFileSystem::FindClose(ZafFileInfoStruct &fileInfo)
{
	int tError = -1;
#if defined(ZAF_WIN32)
	HANDLE *searchHandle = (HANDLE *)fileInfo.internalHandle;
 	if (searchHandle)
	{
		::FindClose(*searchHandle);
		delete searchHandle;
		tError = 0;
	}
#elif defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__)
 	if (fileInfo.internalHandle)
	{
		delete fileInfo.internalHandle;
		tError = 0;
	}
#elif defined(_MSC_VER)
 	if (fileInfo.internalHandle)
	{
		delete fileInfo.internalHandle;
		tError = 0;
	}
#elif defined(__WATCOMC__) && !defined(__QNX__)
 	if (fileInfo.internalHandle)
	{
		delete fileInfo.internalHandle;
		tError = 0;
	}
#elif defined(ZAF_MOTIF)
	DIR *directory = (DIR *)fileInfo.internalHandle;
 	if (directory)
	{
		closedir(directory);
		tError = 0;
	}
#elif defined(ZAF_MACINTOSH)
 	if (fileInfo.internalHandle)
	{
		delete fileInfo.internalHandle;
		tError = 0;
	}
#endif
 	fileInfo.internalHandle = ZAF_NULLP(void);
	return (tError);
}

int ZafDiskFileSystem::GetCWD(ZafIChar *pathName, int pathLength)
{
	// Get the current working directory.
#if defined(ZAF_MACINTOSH)
	// Not implemented.
#	pragma unused(pathName)
#	pragma unused(pathLength)
	return (-1);
#else
	return (getcwd(pathName, pathLength) ? 0 : -1);
#endif
}

int ZafDiskFileSystem::MkDir(const ZafIChar *pathName, ZafStringID ,
	ZafNumberID)
{
	char osName[ZAF_MAXPATHLEN];
	zafCodeSet->ConvertToOSString(pathName, osName, false);

#if defined(ZAF_POSIX)
	return (mkdir(osName, 0777));
#elif defined(ZAF_MACINTOSH)
	long createdDirID;
	return (DirCreate(0, 0L, c2pstr(osName), &createdDirID));
#else
	return (mkdir(osName));
#endif
}

void ZafDiskFileSystem::Close(ZafFile *file)
{
	ZafDiskFile *diskFile = (ZafDiskFile *)file;
	delete diskFile;
}

void ZafDiskFileSystem::DeleteDriveNames(ZafIChar *driveName[])
{
	if (driveName)
	{
		for (int index = 0; driveName[index]; index++)
			delete []driveName[index];
		delete []driveName;
	}
}

#if defined(ZAF_X11) || defined(ZAF_MACINTOSH)
int ZafDiskFileSystem::GetDriveNames(ZafIChar **getDriveName[])
{
	// Motif does not have drives, and the Macintosh Toolbox
	// does not provide a routine for getting the mounted drive names.
	*getDriveName = 0;
	return (-1);
}
#else
int ZafDiskFileSystem::GetDriveNames(ZafIChar **getDriveName[])
{
#if defined(_MSC_VER)
	int currentDrive = _getdrive() + 1;
	int drives = 0;
	bool available[26];

	// Assume drives 0 and 1 (a, b) are available.  This prevents us from
	// querying them, which may cause the system to display an error
	// about there not being a disk in the drive.
	available[0] = available[1] = true;
	drives = 2;
	for (int drive = 2; drive < 26; drive++)
	{
		if (_chdrive(drive + 1) == 0)
		{
			drives++;
			available[drive] = true;
		}
		else
			available[drive] = false;
	}
	_chdrive(currentDrive);

	if (getDriveName)
	{
		ZafIChar **driveName = new ZafIChar *[drives + 1];
		int availableIndex = 0;
		for (int index = 0; index < drives; index++)
		{
			while (!available[availableIndex++])
				;
			driveName[index] = new ZafIChar[3];
			sprintf(driveName[index], ZAF_ITEXT("%c:"), 'A' + availableIndex - 1);
		}
		driveName[drives] = ZAF_NULLP(ZafIChar);

		*getDriveName = driveName;
	}
	return (drives);
#elif defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__)
	int currentDrive = getdisk() + 1;
	int drives;
	bool available[26];

	// Assume drives 0 and 1 (a, b) are available.  This prevents us from
	// querying them, which may cause the system to display an error
	// about there not being a disk in the drive.
	available[0] = available[1] = true;
	drives = 2;
	for (int drive = 2; drive < 26; drive++)
	{
		if (_chdrive(drive + 1) == 0)
		{
			drives++;
			available[drive] = true;
		}
		else
			available[drive] = false;
	}
	_chdrive(currentDrive);

	if (getDriveName)
	{
		ZafIChar **driveName = new ZafIChar *[drives + 1];
		int availableIndex = 0;
		for (int index = 0; index < drives; index++)
		{
			while (!available[availableIndex++])
				;
			driveName[index] = new ZafIChar[3];
			sprintf(driveName[index], ZAF_ITEXT("%c:"), 'A' + availableIndex - 1);
		}
		driveName[drives] = ZAF_NULLP(ZafIChar);

		*getDriveName = driveName;
	}
	return (drives);
#elif defined(__WATCOMC__) && !defined(__QNX__)
	unsigned currentDrive, total;
	_dos_getdrive(&currentDrive);
	int drives = 0;
	bool available[26];

	// Assume drives 0 and 1 (a, b) are available.  This prevents us from
	// querying them, which may cause the system to display an error
	// about there not being a disk in the drive.
	available[0] = available[1] = true;
	drives = 2;
	for (int drive = 2; drive < 26; drive++)
	{
		unsigned setDrive, getDrive;
		setDrive = drive + 1;
		_dos_setdrive(setDrive, &total);
		_dos_getdrive(&getDrive);

		if (getDrive == setDrive)
		{
			drives++;
			available[drive] = true;
		}
		else
			available[drive] = false;
	}
	_dos_setdrive(currentDrive, &total);

	if (getDriveName)
	{
		ZafIChar **driveName = new ZafIChar *[drives + 1];
		int availableIndex = 0;
		for (int index = 0; index < drives; index++)
		{
			while (!available[availableIndex++])
				;
			driveName[index] = new ZafIChar[3];
			sprintf(driveName[index], ZAF_ITEXT("%c:"), 'A' + availableIndex - 1);
		}
		driveName[drives] = ZAF_NULLP(ZafIChar);

		*getDriveName = driveName;
	}
	return (drives);
#else
	//??? Not implemented yet.
	return (-1);
#endif
}
#endif

ZafFile *ZafDiskFileSystem::Open(const ZafIChar *fileName,
	const ZafFileMode mode, ZafStringID, ZafNumberID)
{
	ZafDiskFile *file = new ZafDiskFile(fileName, mode);
	return file;
}

int ZafDiskFileSystem::Remove(const ZafIChar *name)
{
	char osName[ZAF_MAXPATHLEN];
	zafCodeSet->ConvertToOSString(name, osName, false);

#if defined(ZAF_MACINTOSH)
	return (HDelete(0, 0L, c2pstr(osName)) == noErr ? 0 : -1);
#else
	return (remove(osName));
#endif
}

int ZafDiskFileSystem::Rename(const ZafIChar *oldName, const ZafIChar *newName,
	ZafStringID, ZafNumberID)
{
	char osOldName[ZAF_MAXPATHLEN], osNewName[ZAF_MAXPATHLEN];
	zafCodeSet->ConvertToOSString(oldName, osOldName, false);
	zafCodeSet->ConvertToOSString(newName, osNewName, false);

#if defined(ZAF_MACINTOSH)
	return (HRename(0, 0L, c2pstr(osOldName), c2pstr(osNewName)) == noErr ? 0 : -1);
#else
	return (::rename(osOldName, osNewName));
#endif
}

int ZafDiskFileSystem::RmDir(const ZafIChar *pathName, bool deleteContents)
{
	char osPath[ZAF_MAXPATHLEN];
	zafCodeSet->ConvertToOSString(pathName, osPath, false);

	if (deleteContents)
	{
		ZafIChar currentDirectory[ZAF_MAXPATHLEN];
		GetCWD(currentDirectory, ZAF_MAXPATHLEN);
		ChDir(pathName);

		ZafFileInfoStruct fileInfo;
		int finished = FindFirst(fileInfo, ZAF_ITEXT("*"));
		while (!finished)
		{
			if (fileInfo.Directory())
				RmDir(fileInfo.name, true);
			else
				Remove(fileInfo.name);
			finished = FindNext(fileInfo, ZAF_ITEXT("*"));
		}

		ChDir(currentDirectory);
	}

	int result = 0;
#if defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__) || defined(__WATCOMC__) || defined(ZAF_X11)
	if (rmdir(osPath) != 0)
		result = -1;
#elif defined(_MSC_VER)
	if (_rmdir(osPath) != 0)
		result = -1;
#elif defined(ZAF_MACINTOSH)
	if (HDelete(0, 0L, c2pstr(osPath)) != noErr)
		result = -1;
#else
	//??? Not Implemented yet.
#endif
	return (result);
}

#if defined(ZAF_X11) || defined(ZAF_MACINTOSH)
int ZafDiskFileSystem::SetDrive(const ZafIChar *)
{
	// Motif and Macintosh do not have the concept of a current drive.
	return (-1);
}
#else
int ZafDiskFileSystem::SetDrive(const ZafIChar *driveName)
{
#if defined(__BCPLUSPLUS__) || defined(__TCPLUSPLUS__) || defined(_MSC_VER)
	return (_chdrive(toupper(*driveName) - 'A' + 1));
#elif defined(__WATCOMC__) && !defined(__QNX__)
	unsigned setDrive = toupper(*driveName) - 'A' + 1;
	unsigned getDrive = 0;
	unsigned drives;
	_dos_setdrive(setDrive, &drives);
	_dos_getdrive(&getDrive);
	return ((getDrive == setDrive) ? 0 : -1);
#else
	//??? Not implemented yet.
	return (-1);
#endif
}
#endif

bool ZafDiskFileSystem::ValidName(const ZafIChar *name, bool create)
{
#if defined(ZAF_MACINTOSH)
	// First, try a simple open-then try for create mode.
	short tFile = MacOpen(name, ZafDiskFile::fileCreator, ZafDiskFile::fileType, ZAF_FILE_READ);
	if (tFile)
	{
		FSClose(tFile);
		return (true);
	}
	else if (create)
	{
		tFile = MacOpen(name, ZafDiskFile::fileCreator, ZafDiskFile::fileType, ZAF_FILE_OPENCREATE);
		if (tFile)
		{
			char osName[ZAF_MAXPATHLEN];
			zafCodeSet->ConvertToOSString(name, osName, false);
			FSClose(tFile);
			HDelete(0, 0L, c2pstr(osName));
			return (true);
		}
	}
#else
	// First, try a simple open-then try for create mode.
	FILE *tFile = fopen(name, ZAF_ITEXT("r"));
	if (tFile)
	{
		fclose(tFile);
		return (true);
	}
	else if (create)
	{
		tFile = fopen(name, ZAF_ITEXT("a"));
		if (tFile)
		{
			fclose(tFile);
			unlink(name);
			return (true);
		}
	}
#endif

	// Failure.
	return (false);
}

// ----- ZafDiskFile --------------------------------------------------------

ZafDiskFile::ZafDiskFile(const ZafIChar *fileName, ZafFileMode newMode) :
	ZafFile(newMode),
	file(0)
{
	// Set the pathname.
	if (fileName)
#if defined(ZAF_MOTIF)
	{
		ZafIChar *tilde = strchr(fileName, '~');
		if (tilde)
		{
			int tildeIndex = tilde - fileName;
			ZafIChar *home = getenv(ZAF_ITEXT("HOME"));
			pathName = new ZafIChar[strlen(fileName) + strlen(home)];
			strncpy(pathName, fileName, tildeIndex);
			pathName[tildeIndex] = 0;

			ZafIChar *appendPath = tilde + 1;
			if (*appendPath == zafCodeSet->dirSepStr[0])
				appendPath++;
			ZafDiskFileSystem::MakeFullPath(pathName, pathName, home);
			ZafDiskFileSystem::MakeFullPath(pathName, pathName, appendPath);
		}
		else
			pathName = strdup(fileName);
	}
#else
		pathName = strdup(fileName);
#endif
	else
		pathName = ZAF_NULLP(ZafIChar);

	ZafIChar tStringID[ZAF_MAXNAMELEN];
	ZafDiskFileSystem::StripFullPath(pathName, ZAF_NULLP(ZafIChar), tStringID);
	SetStringID(tStringID);

	// Check the mode.
	if (DerivedAccess())
		return;
	else if (!fileName)
	{
		error = ZAF_ERROR_INVALID_NAME;
		return;
	}

#if defined(ZAF_MACINTOSH)
	file = MacOpen(pathName, fileCreator, fileType, mode);
#else
	// Determine the mode.
	ZafIChar openMode[6];
	if (OpenCreate())
		strcpy(openMode, ZAF_ITEXT("a+"));
	else if (Create())
		strcpy(openMode, ZAF_ITEXT("w+"));
	else if (ReadWrite())
		strcpy(openMode, ZAF_ITEXT("r+"));
	else
		strcpy(openMode, ZAF_ITEXT("r"));

	if (BinaryMode())
		strcat(openMode, ZAF_ITEXT("b"));
	else if (TextMode())
		strcat(openMode, ZAF_ITEXT("t"));

	// Try first for a direct open.
	file = fopen(pathName, openMode);
#endif

	// Next, try a path open.
	if (!file && zafSearchPath && !strchr(pathName, ':') && !strchr(pathName, zafCodeSet->dirSepStr[0]))
	{
		ZafIChar fullPath[ZAF_MAXPATHLEN];
		for (ZafPathElement *element = zafSearchPath->First(); !file && element; element = element->Next())
		{
			ZafDiskFileSystem::MakeFullPath(fullPath, element->Path(), fileName);
#			if defined(ZAF_MACINTOSH)
			file = MacOpen(fullPath, fileCreator, fileType, mode);
#			else
			file = fopen(fullPath, openMode);
#			endif
			if (file)
			{
				delete pathName;
				pathName = strdup(fullPath);
			}
		}
	}

	// Temporary mode setting functions.
#	if defined(ZAF_MOTIF)
	if (file && (Create() || OpenCreate()))
	{
#		if defined(ZAF_UNICODE)
		char *osPath = zafCodeSet->ConvertToOSString(pathName);
		chmod(osPath, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
		delete []osPath;
#		else
		chmod(pathName, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
#		endif
	}
#	endif

	// Set the error based on the file status.
	if (!file)
		error = ZAF_ERROR_FILE_OPEN;
}

ZafDiskFile::~ZafDiskFile(void)
{
	// Delete the file.
	if (!DerivedAccess() && file)
#if defined(ZAF_MACINTOSH)
		FSClose(file);
#else
		fclose(file);
#endif

	// Delete the pathname.
	if (pathName)
		delete []pathName;
}

ZafOffset ZafDiskFile::Length(void) const
{
	// Return the file length.
	ZafOffset position = -1;
	if (file)
	{
#if defined(ZAF_MACINTOSH)
		GetEOF(file, &position);
#else
		long currentPosition = ftell(file);
		fseek(file, 0, ZAF_SEEK_END);
		position = ftell(file);
		fseek(file, currentPosition, ZAF_SEEK_START);
#endif
	}
	return (position);
}

int ZafDiskFile::Seek(ZafOffset offset, ZafSeek location)
{
	// Seek into the file.
	int result = -1;
	if (file)
	{
#if defined(ZAF_MACINTOSH)
		short fromLoc = fsFromStart;
		switch (location)
		{
		case ZAF_SEEK_START:
			fromLoc = fsFromStart;
			break;
		case ZAF_SEEK_CURRENT:
			fromLoc = fsFromMark;
			break;
		case ZAF_SEEK_END:
			fromLoc = fsFromLEOF;
			break;
		}
		result = SetFPos(file, fromLoc, offset);
#else
		result = fseek(file, offset, (int)location);
#endif
	}

	if (result)
		error = ZAF_ERROR_FILE_POSITION;
	return (result);
}

ZafOffset ZafDiskFile::Tell(void) const
{
	// Tell the current offset.
	ZafOffset position = -1;
	if (file)
#if defined(ZAF_MACINTOSH)
		GetFPos(file, &position);
#else
		position = ftell(file);
#endif
	return (position);
}

// ----- I/O ----------------------------------------------------------------

int ZafDiskFile::ReadData(void *buffer, int size)
{
#if defined(ZAF_MACINTOSH)
	long count = size;
	OSErr theError = FSRead(file, &count, (Ptr)buffer);
	if (theError == noErr || theError == eofErr)
		return ((int)count);
	else
	{
		error = ZAF_ERROR_FILE_READ;
		return (0);
	}
#else
	int result = fread((char *)buffer, 1, size, file);
	if (ferror(file))
		error = ZAF_ERROR_FILE_READ;
	return (result);
#endif
}

int ZafDiskFile::WriteData(const void *buffer, int size)
{
	// Check for a valid size.
	if (size == 0)
		return (size);

#if defined(ZAF_MACINTOSH)
	long count = size;
	if (FSWrite(file, &count, (Ptr)buffer) == noErr)
	{
		if (count != (long)size)
			error = ZAF_ERROR_FILE_WRITE;
		return ((int)count);
	}
	else
	{
		error = ZAF_ERROR_FILE_WRITE;
		return (0);
	}
#else
	int result = fwrite((char *)buffer, 1, size, file);
	if (result != size)
		error = ZAF_ERROR_FILE_WRITE;
	return (result);
#endif
}

